pub struct BearerTokenAuthorizer { /* private fields */ }Expand description
Create a simple Authorizer that attaches a given token to any request
a client sends. The token is attached with the Bearer auth-scheme.
Tonic
If the tonic feature is enabled, tonic::service::Interceptor is implemented for
BearerTokenAuthorizer. The interceptor does not insert the access token if the intercepted call
already has an Authorization header.
Implementations